-- hds header_start
--
-- VHDL Architecture CERC_be_fpga.be_trig_top_tester.sim
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 10:48:52 09/22/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity be_trig_top_tester is
   port( 
      fe_trig            : in     std_logic_vector (7 downto 0);
      trig_ser_config_rx : in     std_logic;
      clk                : out    std_logic;
      rst                : out    std_logic;
      trig_ser_config_tx : out    std_logic;
      trig_ext0          : out    std_logic;
      trig_ext1          : out    std_logic
   );

-- Declarations

end be_trig_top_tester ;


-- hds interface_end

architecture test of be_trig_top_tester is

	constant CLKPERIOD 	: time := 25 ns;

	signal 	clk_osc 		: std_logic := '1';

	signal 	ser_tx 		: std_logic := '0';
	signal 	ser_rx		: std_logic := '0';
	
	signal 	rx_data		: std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	
	constant	SREAD			: std_logic := '1';
	constant	SWRITE		: std_logic := '0';
	constant SLENGTH		: std_logic_vector(15 downto 0) := "0000000000100000"; -- 32
	
begin

	-- clock generator
	------------------
	clk_osc <= not clk_osc after CLKPERIOD / 2;
	clk <= clk_osc;
	
	-- IO Assignments
	-----------------
   trig_ser_config_tx <= ser_tx;
	ser_rx <= trig_ser_config_rx;

	
	-----------------------------------------------------------------------
	main_sim: process

		-- Procedures
		----------------------------------------------------------------
		procedure Serial_Read(iss_desig, iss_addr : in integer ) is
		
			variable	ss_desig : std_logic_vector(4 downto 0);
			variable	ss_addr	: std_logic_vector(15 downto 0);
			
			--variable SLENGTH_PLUS1 : std_logic_vector(15 downto 0);
			
		begin
			ss_desig := CONV_STD_LOGIC_VECTOR(iss_desig, 5);
			ss_addr  := CONV_STD_LOGIC_VECTOR(iss_addr, 16);
			--SLENGTH_PLUS1 := SLENGTH + '1';
			
			wait until rising_edge(clk_osc); wait for 1 ns;

			ser_tx <= '1'; 					-- start bit
			wait until rising_edge(clk_osc); wait for 1 ns;
		
			ser_tx <= SREAD; 					-- read/write bit
			wait until rising_edge(clk_osc); wait for 1 ns;

			for n in 4 downto 0 loop		-- designator (4:0)
				ser_tx <= ss_desig(n);
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;

			ser_tx <= '1'; 					-- length start bit
			wait until rising_edge(clk_osc); wait for 1 ns;

			for n in 15 downto 0 loop		-- length (15:0)
				--ser_tx <= SLENGTH_PLUS1(n);
				ser_tx <= ss_addr(n);
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;
						
			ser_tx <= '1'; 					-- 'stop' bit
			wait until rising_edge(clk_osc); wait for 1 ns;
			ser_tx <= '0';
		

			-- Get returned data
			------------------------------------
			--for n in 1 to 3 loop				-- rx data latency
				wait until rising_edge(clk_osc);  wait for 1 ns;
			--end loop;
			
			for n in 31 downto 0 loop		-- rx data
				wait for 5 ns;
				rx_data(n) <= ser_rx;
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;
			
		end procedure;
		----------------------------------------------------------------

		procedure Serial_Write( iss_desig, iss_addr, iss_data : in integer ) is
		
			variable	ss_desig : std_logic_vector(4 downto 0);
			variable	ss_addr	: std_logic_vector(15 downto 0);
			variable	ss_data	: std_logic_vector(31 downto 0);									
		begin
			ss_desig := CONV_STD_LOGIC_VECTOR(iss_desig, 5);
			ss_addr  := CONV_STD_LOGIC_VECTOR(iss_addr, 16);
			ss_data	:= CONV_STD_LOGIC_VECTOR(iss_data, 32);
			
			wait until rising_edge(clk_osc); wait for 1 ns;
					
			ser_tx <= '1'; 					-- start bit
			wait until rising_edge(clk_osc); wait for 1 ns;
						
			ser_tx <= SWRITE; 				-- read/write bit
			wait until rising_edge(clk_osc); wait for 1 ns;

			for n in 4 downto 0 loop		-- designator (4:0)
				ser_tx <= ss_desig(n);
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;

			ser_tx <= '1'; 					-- addr start bit (ex length field)
			wait until rising_edge(clk_osc); wait for 1 ns;

			for n in 15 downto 0 loop		-- addr (15:0) (was length field)
				ser_tx <= ss_addr(n);
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;
			
			for n in 31 downto 0 loop		-- data
				ser_tx <= ss_data(n);
				wait until rising_edge(clk_osc); wait for 1 ns;
			end loop;

			ser_tx <= '0'; 					-- idle
			wait until rising_edge(clk_osc); wait for 1 ns;
			wait until rising_edge(clk_osc); wait for 1 ns;

		end procedure;
		---------------------------------------------------------------
		procedure RegRead(iss_desig : in integer ) is
		begin
			Serial_Read(iss_desig, 0);
		end procedure;
		---------------------------------------------------------------
		procedure RegWrite(iss_desig, iss_data : in integer ) is
		begin
			Serial_Write(iss_desig, 0, iss_data);
		end procedure;
		---------------------------------------------------------------
		procedure RAMRead(iss_addr : in integer ) is
		begin
			Serial_Read(0, iss_addr);
		end procedure;
		---------------------------------------------------------------
		procedure RAMWrite(iss_addr, iss_data : in integer ) is
		begin
			Serial_Write(0, iss_addr, iss_data);
		end procedure;
		---------------------------------------------------------------
		
	----------------------------------------------------------------------
	-- Main Simulation
	----------------------------------------------------------------------
	begin
		
		wait for 1 ns;
	
		-- Initial Values
		trig_ext0 <= '0';
		trig_ext1 <= '0';
		rst <= '1';
		
		-- Reset
		rst <= '1';
		wait for 100 ns;
		rst <= '0';
		wait for 3000 ns;
		
		trig_ext0 <= '1';
		wait for 10 ns;
		trig_ext0 <= '0';
		wait for 15 ns;
		
		wait for 100 ns;
		RegWrite(1,1);		-- Enable trig-ext0
		wait for 2000 ns;
				
		trig_ext0 <= '1';
		wait for 10 ns;
		trig_ext0 <= '0';
		wait for 15 ns;

		
		wait for 200 ns;
		RegRead(11);

		wait for 200 ns;
		RegWrite(11,0);
		
		wait for 400 ns;
		trig_ext0 <= '1';
		wait for 200 ns;

		RegRead(11);
		wait for 2000 ns;
	
		RegWrite(11,0);
		wait for 2000 ns;

		RegRead(11);
		wait for 2000 ns;


		

		--RegWrite(2,16#80000200#);
		--RegWrite(3,16#80000333#);				
			
		--RegRead(2);
		--RegRead(3);				
		
		--wait for 200 ns;
		
		--RegWrite(31,16#80000100#);
		--RegWrite(30,16#00000001#);
		--RegWrite(30,16#00000002#);
		--RegWrite(30,16#00000003#);
		--RegWrite(30,16#00000004#);
		--RegWrite(30,16#00000005#);
		--RegWrite(30,16#00000006#);
		
		--wait for 200 ns;
		
		--RegWrite(31,16#80000100#);
		--RegRead(30);
		--RegRead(30);
		--RegRead(30);
		--RegRead(30);
		--RegRead(30);
		--RegRead(30);
	
		--RamWrite(16#0004#, 16#12345678#);
		--RamWrite(16#0100#, 16#ABCDEF09#);
		
		--RamRead(16#0004#);
		--RamRead(16#0100#);
			
		
		wait;      
  	end process;  
end test;
