-- hds header_start
--
-- VHDL Architecture CALICE_seriface.protocol.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 21:33:41 11/02/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity vmeser_decode is
   port( 
      clk      : in     std_logic;
      rst      : in     std_logic;
      ser_i    : in     std_logic;
      ser_o    : out    std_logic;
      db_rd0_o : out    std_logic; -- 1 clk early read - for FIFOs mainly
      db_rd_o  : out    std_logic;
      db_wr_o  : out    std_logic;
      ren_o    : out    std_logic_vector (31 downto 0); -- reg enable lines (like reg_en)
      db_io    : inout  std_logic_vector (31 downto 0)
   );

-- Declarations

end vmeser_decode ;

-- hds interface_end


-- The serial protocol
-- ===================
--  Write: 1-0-DESIG-1-LENGTHxxxxxxxxxx-DATAxxxxxxxxxxxxxxxxxxxxxxxxxxxx
--  Read:  1-1-DESIG-1-LENGTHxxxxxxxxxx-1
--         1+1+5    +1+16              +32 =                          64


architecture rtl of vmeser_decode is

	constant ZERO32 	: std_logic_vector(31 downto 0) := "00000000000000000000000000000000";
	constant ZERO16 	: std_logic_vector(15 downto 0) := "0000000000000000";
							
	-- Note: initials are used here for no good reason anymore! But they don't harm.
	signal data_in_buff 	: std_logic_vector(31 downto 0) := ZERO32;
	signal data_out_buff : std_logic_vector(31 downto 0) := ZERO32;
	
	signal sc_rd_nwr		: std_logic;
	signal sc_desig 		: std_logic_vector(4 downto 0) := "00000";
	signal sc_length		: std_logic_vector(15 downto 0) := ZERO16;
	
begin

	---------------------------------------------------------------------------
	-- Serial Machine
	---------------------------------------------------------------------------

	-- The serial protocol:
	--    0	 	-  Desig Start Bit
	--	   1   	-  Read_nWrite	
	--    2-6	-  Designator
	--		7		-  Lenth Start Bit
	--		8-23  -  Length
	--		24-55 -  Write Data       
	-- OR 24 	-  Stop Bit (ignored)

	machine1 : process (clk)
		variable state : integer range 0 to 63;

	begin
		if rising_edge(clk) then
			if (rst = '1') then
				state := 0;
				
				sc_rd_nwr 		<= '0';
				sc_desig  		<= (others => '0');
				sc_length		<= (others => '0');

				data_in_buff 	<= (others => '0');
				data_out_buff 	<= (others => '0');
				
				db_rd0_o			<= '0';
				db_rd_o 			<= '0';
				db_wr_o 			<= '0';
				db_io	  			<= (others => 'Z');
				
			else
				--defaults
				db_rd0_o	<= '0';				
				db_rd_o 	<= '0';
				db_wr_o 	<= '0';
				db_io	  	<= (others => 'Z');

				-- wait for start of data
				if (state = 0) then
					-- default
					state := 0;
					
					if (ser_i = '1') then
						state := state + 1;
					end if;

				-- Read/nWrite bit
				elsif (state = 1) then
					sc_rd_nwr <= ser_i after 1 ns;
					
					state := state + 1;
					
				-- Designator (5 bits)
				elsif (state < 7) then
					sc_desig <= (sc_desig(3 downto 0) & ser_i) after 1 ns;
					
					state := state + 1;
				
				-- Length/Address Start bit
				elsif (state = 7) then
					
					state := state + 1;
						
				-- Length/Address (16 bits)
				elsif (state < 23) then
					sc_length <= (sc_length(14 downto 0) & ser_i) after 1 ns;
					
					state := state + 1;
				
				-- issue early read for things like FIFOs
				elsif (state = 23) then
					sc_length <= (sc_length(14 downto 0) & ser_i) after 1 ns;
					
					if (sc_rd_nwr = '1') then
						db_rd0_o <= '1' after 1 ns;
					end if;

					state := state + 1;
					
				-- Write cycle SHIFT 0/Read cycle START
				elsif (state = 24) then
					data_in_buff <= (data_in_buff(30 downto 0) & ser_i) after 1 ns; -- do this anyway, in case it's a write cycle
								
					if (sc_rd_nwr = '1') then
						db_rd_o <= '1' after 1 ns;
					end if;
					
					state := state + 1;

				-- Write cycle SHIFT 1/Read cycle LOAD
				elsif (state = 25) then
					data_in_buff <= (data_in_buff(30 downto 0) & ser_i) after 1 ns;
					
					if (sc_rd_nwr = '1') then
						data_out_buff <= db_io after 1 ns;

					end if;

					state := state + 1;
					
				-- Write cycle SHIFT 2-31/Read cycle SHIFT 0-29
				elsif (state < 56) then
					data_in_buff  <= (data_in_buff(30 downto 0) & ser_i) after 1 ns;

					if (sc_rd_nwr = '1') then
						--ser_o <= data_out_buff(31);
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0') after 1 ns;
					end if;

					state := state + 1;
				
				-- Write cycle STROBE/Read cycle SHIFT 30
				elsif (state = 56) then
					
					if (sc_rd_nwr = '1') then
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0') after 1 ns;

					else
						db_wr_o <= '1' after 1 ns;
						db_io   <= data_in_buff after 1 ns;

					end if;

					state := state + 1;
					
				-- Write cycle IDLE/ Read cycle SHIFT 31
				elsif (state = 57) then
					
					if (sc_rd_nwr = '1') then
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0') after 1 ns;
					end if;

					state := 0;
				
				else
					state := 0;
					
				end if;
			end if;
		end if;
	end process;

	ser_o <= data_out_buff(31) after 1 ns;


	---------------------------------------------------------------------------
	-- Designator - 5 to 32 Decoder
	---------------------------------------------------------------------------

	process (clk, sc_desig)
		variable idesig : integer range 0 to 16#1F#; -- 5 bit
		
	begin
  	
  		idesig := CONV_INTEGER(sc_desig);
  		
  		if rising_edge(clk) then
  			if (rst = '1') then
  				ren_o <= (others => '0');
  		
	    	else
	    		-- default
	    		ren_o <= (others => '0') after 1 ns;
	    	
	    		-- assign reg-line
				ren_o(idesig) <= '1' after 1 ns;

    		end if;
		end if;
  	end process;

end rtl;
