-- hds header_start
--
-- VHDL Architecture CERC_trig.fifo_control.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 11:22:06 10/24/05
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity fifo_control is
   generic( 
      FIFO_DEPTH : integer := 256
   );
   port( 
      rst              : in     std_logic;
      clk              : in     std_logic;
      db_rd0_i         : in     std_logic;
      db_rd_i          : in     std_logic;
      db_oz            : out    std_logic_vector (31 downto 0);
      ren_fifo_stat_i  : in     std_logic;
      ren_fifo_i       : in     std_logic;
      squirt_en_i      : in     std_logic;
      fifo_idlelvl_i   : in     std_logic_vector (15 downto 0);
      fifo_testmode_i  : in     std_logic_vector (1 downto 0);
      tqdr_en_i        : in     std_logic;
      tqdr_fifo_rd_i   : in     std_logic;
      busy_trig_i      : in     std_logic;
      f1_din_o         : out    std_logic_vector (31 downto 0);
      f1_dout_i        : in     std_logic_vector (31 downto 0);
      f1_rd_o          : out    std_logic;
      f1_wr_o          : out    std_logic;
      f1_empty_i       : in     std_logic;
      f1_full_i        : in     std_logic;
      f1_count_i       : in     std_logic_vector (7 downto 0);
      f1_init_o        : out    std_logic;
      fifo_data_o      : out    std_logic_VECTOR (31 downto 0);
      fifo_data_i      : in     std_logic_VECTOR (31 downto 0);
      fifo_empty_o     : out    std_logic;
      fifo_full_o      : out    std_logic;
      squirt_disable_o : out    std_logic;
      trig_id_i        : in     std_logic_VECTOR (31 downto 0);
      fifo_max_depth_o : out    std_logic_vector (9 downto 0)
   );

-- Declarations

end fifo_control ;


-- hds interface_end

architecture rtl of fifo_control is

   signal busy_trig_q 		: std_logic;
   signal busy_falling 		: std_logic;
   signal db_fifo_rd  	 	: std_logic;
   signal depth_hold   		: std_logic;
   signal rotate       		: std_logic;
   signal db_fifo_rd_q    	: std_logic;
   signal fifo_stat_rd 		: std_logic;
   --signal fifo_full   		: std_logic;
   signal tqdr_rd   			: std_logic;
	signal reg_fifo_stat		: std_logic_vector(31 downto 0);
	signal fifo_data_out			: std_logic_vector(31 downto 0);
	
	signal count_pattern		: std_logic_vector(31 downto 0);
	signal osc_pattern		: std_logic_vector(31 downto 0);
	
	
	constant CONST_PATTERN  : std_logic_vector(31 downto 0) 
		:= "11110001111000101101001111000100"; -- 0xf1e2d3c4
	
	constant OSC_PATTERN0 	: std_logic_vector(31 downto 0) 
		:= "01010101010101010101010101010101"; -- 0x55555555
		
	constant OSC_PATTERN1 	: std_logic_vector(31 downto 0) 
		:= "10101010101010101010101010101010"; -- 0xAAAAAAAA	
		
	constant HEADER			: std_logic_vector(31 downto 0) 
		:= "00010010001101000101011001111000"; -- 0x12345678
	
	constant TRAILER			: std_logic_vector(31 downto 0) 
		:= "11111110110111001011101010011000"; -- 0xfedcba98
		

begin

	fifo_empty_o 		<= f1_empty_i;
	
	squirt_disable_o <= f1_empty_i or tqdr_en_i;
	
	
	--fifo_full <= '1' when ( (f1_full_i = '1') or (f1_count_i = (FIFO_MAX_DEPTH-1)) ) else '0';
	fifo_full_o  		<= f1_full_i;
	
	fifo_max_depth_o 	<= conv_std_logic_vector(FIFO_DEPTH, 10);
    
     
   -- FIFO READ control
   -----------------------------------
   											-- > used here to stop fifo runaway post reset
	depth_hold 	<= '1' when (f1_count_i > fifo_idlelvl_i(7 DOWNTO 0) ) else '0';
	
	rotate 		<= depth_hold and not(busy_trig_i);
	tqdr_rd 		<= tqdr_en_i and tqdr_fifo_rd_i;
	
	f1_rd_o 		<= rotate or (db_fifo_rd and not(tqdr_en_i)) or tqdr_rd;
	
	
	
  	fifo_data_out	<= HEADER 		when (f1_count_i = FIFO_DEPTH-1)	else
   						trig_id_i 	when (f1_count_i = FIFO_DEPTH-2) else
   						TRAILER 		when (f1_empty_i = '1')				else
   						f1_dout_i;
   						
   fifo_data_o <= fifo_data_out;

     
   -- FIFO reset (on falling busy)
   ----------------------------------------
   
   edgedet : process(clk)
   begin
	   if rising_edge(clk) then
	   	if (rst = '1') then
	   		busy_trig_q <= '0';
	   		
	   	else
	         busy_trig_q <= busy_trig_i;
	         
	      end if;
	   end if;
	end process;
	
	-- *** this isn't a good idea for fast trigger rates (PD)
	busy_falling <= not(busy_trig_i) and busy_trig_q;   
	f1_init_o <= busy_falling or rst;
     
	
	-- FIFO Write control
   -----------------------------------
        
   process (clk)
   begin
      if rising_edge(clk) then
         if (rst = '1') then
            f1_wr_o <= '0';
         
         elsif (f1_full_i = '1') then
         	f1_wr_o <= '0';
         	
         elsif (not(busy_trig_i) = '1') then
         	f1_wr_o <= '1';
         	
         end if;
   	end if;
   end process;
	
			
	pattern_gens : process (clk)
   begin
      if rising_edge(clk) then
         if (rst = '1') or (busy_falling = '1') then
            count_pattern  <= (others => '0');
            osc_pattern    <= OSC_PATTERN0;
                     
         else
         	count_pattern  <= count_pattern + '1';
         	
         	if (count_pattern(0) = '1') then
         		osc_pattern <= OSC_PATTERN0;
         	else
         		osc_pattern <= OSC_PATTERN1;
         	end if;
         end if;
   	end if;
   end process;
	
	
	-- Assign data to data on it's way INTO the fifo:
	with fifo_testmode_i select
		 f1_din_o <=	count_pattern		when "11",
		  					osc_pattern			when "10",
		 					CONST_PATTERN		when "01", 					
		  					fifo_data_i			when others; -- "00"


   -- Data-bus Stuff
   ---------------------------------------------------------------------------   
  
   -- Read FIFO Data
   db_fifo_rd <= (ren_fifo_i or squirt_en_i) and db_rd0_i;
   
   process(clk)
   begin
	   if rising_edge(clk) then
	   	if (rst = '1') then
	   		db_fifo_rd_q <= '0';
	   		
	   	else
	         db_fifo_rd_q <= db_fifo_rd;
	      end if;
	   end if;
	end process;

   db_oz <= fifo_data_out when (db_fifo_rd_q = '1') else (others => 'Z');
   
   -- Read FIFO Status
   fifo_stat_rd <= ren_fifo_stat_i and db_rd_i;
	 
   reg_fifo_stat(31 downto 18) 	<= (others => '0');
   reg_fifo_stat(          17) 	<= f1_full_i;
   reg_fifo_stat(          16)	<= f1_empty_i;
   reg_fifo_stat(15 downto 10) 	<= (others => '0');
   reg_fifo_stat( 9 downto  0) 	<= "00" & f1_count_i;

  	db_oz <= reg_fifo_stat when (fifo_stat_rd = '1') else (others => 'Z');

end rtl;
