-- hds header_start
--
-- VHDL Architecture TIM_fpga2.seq_control.rtl2
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 12:52:13 02/03/04
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity seq_control is
   generic( 
      L : integer := 10
   );
   port( 
      rst             : in     std_logic;
      clk             : in     std_logic;
      db              : inout  std_logic_vector (31 downto 0);
      db_wr           : in     std_logic;
      db_rd           : in     std_logic;
      reg_sequencer_i : in     std_logic_vector (31 downto 0);
      ren_ram_i       : in     std_logic;
      addr_i          : in     std_logic_vector ((L-1) downto 0);
      ram_data_o      : out    std_logic_vector (31 downto 0);
      ram_data_i      : in     std_logic_vector (31 downto 0);
      ram_addr_o      : out    std_logic_vector ((L-1) downto 0);
      ram_wr_o        : out    std_logic;
      ram_en_o        : out    std_logic;
      seq_busy_o      : out    std_logic;
      trig_seq_o      : out    std_logic_vector (15 downto 0);
      busy_trig_i     : in     std_logic
   );

-- Declarations

end seq_control ;


-- hds interface_end

architecture rtl of seq_control is

   signal cyclic_en 		: std_logic;
   signal seq_reset		: std_logic;
   signal seq_go     	: std_logic;
   
   signal seq_busygo_en : std_logic;
   signal seq_start  	: std_logic;
	signal busy				: std_logic;
   signal done   			: std_logic;

	signal seq_count  	: std_logic_vector((L-1) downto 0);
	signal seq_end	   	: std_logic_vector((L-1) downto 0);   

   signal seq_trig	   : std_logic_vector(15 downto 0);   
   signal seq_trig_en   : std_logic_vector(15 downto 0);   
   signal busy_trig_q   : std_logic;
   signal busy_falling  : std_logic;
   

-------------------------------------------------------------------------------
-- MAIN CODE
-------------------------------------------------------------------------------
begin


   seq_reset 		<= reg_sequencer_i(31);
   seq_go    		<= reg_sequencer_i(30);
   seq_busygo_en 	<= reg_sequencer_i(29);
   cyclic_en 		<= reg_sequencer_i(28);
   --reserved		<= reg_sequencer_i(28 downto (LEN_BITS+16));
   seq_end   		<= reg_sequencer_i((L+15) downto 16);
   seq_trig_en 	<= reg_sequencer_i(15 downto  0);
   
   
   -- Generate busy busy_falling
   ----------------------------------------
   
   edgedet : process(clk)
   begin
	   if rising_edge(clk) then
	   	if (rst = '1') then
	   		busy_trig_q <= '0';
	   		
	   	else
	         busy_trig_q <= busy_trig_i;
	         
	      end if;
	   end if;
	end process;

	busy_falling <= not(busy_trig_i) and busy_trig_q;   
   
   seq_start <= seq_go or (seq_busygo_en and busy_falling);


	process (clk)
   begin
   	if rising_edge(clk) then

   		if (rst = '1') or (seq_reset = '1') then
				seq_count <= (others => '0');
				busy  <= '0';
				done  <= '0';
				
   		else
   			if (busy = '1') then
   				if (seq_count = seq_end) then
   					if (cyclic_en = '1') then
   						seq_count <= (others => '0');
   					
   					else
	   					done <= '1';
   						busy <= '0';
   					
   					end if;
   				
   				else
   					seq_count <= seq_count + '1';
   					
   				end if;
   			   				
   			elsif (seq_start = '1') and (done = '0') then
   					busy <= '1';
   			
   			end if;
   		end if;
   	end if;
   end process;
  					

  	seq_busy_o  <= busy;


	ram_en_o   <= ren_ram_i after 1 ns when (busy = '0') else busy  	  after 1 ns;
	ram_wr_o   <= db_wr     after 1 ns when (busy = '0') else '0'  	  after 1 ns;
   ram_addr_o <= addr_i  	after 1 ns when (busy = '0') else seq_count after 1 ns;
	
	
	seq_trig   <= ram_data_i(15 downto 0);

	trig_seq_o <= (seq_trig and seq_trig_en) after 1 ns when (busy = '1') else 
					  (others => '0')            after 1 ns;

   ---------------------------------------------------------------------------
   -- Data Bus
   ---------------------------------------------------------------------------

	ram_data_o <= db;
   
	db <= ram_data_i      after 1 ns when ((db_rd and ren_ram_i) = '1') else 
			(others => 'Z') after 1 ns;
							

   
end rtl;
