-- hds header_start
--
-- VHDL Architecture CALICE_seriface.protocol.rtl
--
-- Created:
--          by - warren.UNKNOWN (ELECTRON)
--          at - 21:33:41 11/02/03
--
-- Generated by Mentor Graphics' HDL Designer(TM) 2002.1b (Build 7)
--
-- hds header_end

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity vmeser_decode is
   port( 
      clk     : in     std_logic;
      rst     : in     std_logic;
      ser_i   : in     std_logic;
      ser_o   : out    std_logic;
      db_rd_o : out    std_logic;
      db_wr_o : out    std_logic;
      addr_o  : out    std_logic_vector (15 downto 0);
      ren_o   : out    std_logic_vector (31 downto 0);
      db_io   : inout  std_logic_vector (31 downto 0)
   );

-- Declarations

end vmeser_decode ;


-- hds interface_end


-- The serial protocol
--  Write: 1-0-DESIG-1-LENGTHxxxxxxxxxx-DATAxxxxxxxxxxxxxxxxxxxxxxxxxxxx
--  Read:  1-1-DESIG-1-LENGTHxxxxxxxxxx-1
--         1+1+5    +1+16              +32 = 


architecture rtl_lenaddr of vmeser_decode is

	signal data_in_buff 	: std_logic_vector(31 downto 0);
	signal data_out_buff : std_logic_vector(31 downto 0);
	
	signal sc_rd_nwr		: std_logic;
	signal sc_desig 		: std_logic_vector(4 downto 0);
	signal sc_addr 		: std_logic_vector(15 downto 0);

begin

	---------------------------------------------------------------------------
	-- Serial Machine
	---------------------------------------------------------------------------

	-- The serial protocol
	--    0	 	-  Desig Start Bit
	--	   1   	-  Read_nWrite	
	--    2-6	-  Designator
	--		7		-  Lenth Start Bit
	--		8-23  -  Length
	--		24-55 -  Write Data       
	-- OR 24 	-  Stop Bit (ignored)

	machine1 : process (clk)
		variable state : integer range 0 to 63;

	begin
		if rising_edge(clk) then
			if (rst = '1') then
				state := 0;
				
				sc_rd_nwr 		<= '0';
				sc_desig  		<= (others => '0');
				sc_addr			<= (others => '0');

				data_in_buff 	<= (others => '0');
				data_out_buff 	<= (others => '0');
				
				db_wr_o 			<= '0';
				db_rd_o 			<= '0';
				db_io	  			<= (others => 'Z');
				
				--ser_o   			<= '0';
				
			else
				--defaults
				db_wr_o <= '0';
				db_rd_o <= '0';
				db_io	  <= (others => 'Z');
				--ser_o   <= '0';

				-- wait for start of data
				if (state = 0) then

					-- default
					state := 0;
					
					if (ser_i = '1') then
						state := state + 1;
					end if;

				-- Read/nWrite bit
				elsif (state = 1) then
					sc_rd_nwr <= ser_i;
					
					state := state + 1;
					
				-- Designator (5 bits)
				elsif (state < 7) then
					sc_desig <= (sc_desig(3 downto 0) & ser_i);
					
					state := state + 1;
				
				-- Length/Address Start bit (ignored)
				elsif (state = 7) then
					
					state := state + 1;
						
				-- Length/Address (16 bits)
				elsif (state < 24) then
					sc_addr <= (sc_addr(14 downto 0) & ser_i);
					
					state := state + 1;
					
				-- Write cycle SHIFT 0/Read cycle START
				elsif (state = 24) then
					data_in_buff <= (data_in_buff(30 downto 0) & ser_i); -- do this anyway, in case it's a write cycle
								
					if (sc_rd_nwr = '1') then
						db_rd_o <= '1';
					end if;
					
					state := state + 1;

				-- Write cycle SHIFT 1/Read cycle LOAD
				elsif (state = 25) then
					data_in_buff <= (data_in_buff(30 downto 0) & ser_i);
					
					if (sc_rd_nwr = '1') then
						data_out_buff <= db_io;

					end if;

					state := state + 1;
				-- Write cycle SHIFT 2-31/ Read cycle SHIFT 0-29
				elsif (state < 56) then
					data_in_buff  <= (data_in_buff(30 downto 0) & ser_i);

					if (sc_rd_nwr = '1') then
						--ser_o <= data_out_buff(31);
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0');
					end if;

					state := state + 1;
				
				-- Write cycle STROBE/Read cycle SHIFT 30
				elsif (state = 56) then
					
					if (sc_rd_nwr = '1') then
						--ser_o <= data_out_buff(30);
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0');

					else
						db_wr_o <= '1';
						db_io   <= data_in_buff;

					end if;

					state := state + 1;
					
				-- Write cycle IDLE/ Read cycle SHIFT 31
				elsif (state = 57) then
					
					if (sc_rd_nwr = '1') then
						--ser_o <= data_out_buff(31);
						data_out_buff(31 downto 0) <= (data_out_buff(30 downto 0) & '0');
					end if;

					state := 0;
				
				else
					state := 0;
					
				end if;
			end if;
		end if;
	end process;

	ser_o <= data_out_buff(31);

	-- Designator to 5 to 32 Decoder
	--------------------------------------------------
	process (clk, sc_desig)
		variable iaddr : integer range 0 to 16#1F#; -- 5 bit
		
	begin
  	
  		iaddr := CONV_INTEGER(sc_desig);
  		
  		if rising_edge(clk) then
  			if (rst = '1') then
  				ren_o <= (others => '0');
  		
	    	else
	    		-- default
	    		ren_o <= (others => '0');
	    	
	    		-- assign reg-line
				ren_o(iaddr) <= '1';

    		end if;
		end if;
  	end process;



	-- Address Bus
	-----------------------------------------------------
	addr_o(15 downto 2) <= sc_addr(15 downto  2);
	addr_o( 1 downto 0) <= "00";


end rtl_lenaddr;


